
function OexSprite_CommonButton() {
    this.initialize(...arguments);
}
function OexSpriteToggleButton() {
    this.initialize(...arguments);
}

(() => {
    const pluginName = "plugin_CommonButton";

    //-----------------------------------------------------------------------------
    // OexSprite_CommonButton
    //

    OexSprite_CommonButton.prototype = Object.create(Sprite_Clickable.prototype);
    OexSprite_CommonButton.prototype.constructor = OexSprite_CommonButton;

    OexSprite_CommonButton.prototype.initialize = function(pictureName, coldFrame, hotFrame) {
        Sprite_Clickable.prototype.initialize.call(this);
        this._clickHandler = null;
        this._coldFrame = coldFrame;
        this._hotFrame = hotFrame;
        this.bitmap = ImageManager.loadPicture(pictureName);
        this.updateFrame();
        this.updateOpacity();
    };

    OexSprite_CommonButton.prototype.update = function() {
        Sprite_Clickable.prototype.update.call(this);
        this.updateFrame();
        this.updateOpacity();
        this.processTouch();
    };

    OexSprite_CommonButton.prototype.updateFrame = function() {
        const frame = this.isPressed() ? this._hotFrame : this._coldFrame;
        if (frame) {
            this.setFrame(frame.x, frame.y, frame.width, frame.height);
        }
    };

    OexSprite_CommonButton.prototype.updateOpacity = function() {
        //this.opacity = this._pressed ? 255 : 192;
    };

    OexSprite_CommonButton.prototype.setClickHandler = function(method) {
        this._clickHandler = method;
    };

    OexSprite_CommonButton.prototype.onClick = function() {
        if (this._clickHandler) {
            this._clickHandler();
        }
    };



    //-----------------------------------------------------------------------------
    // OexSpriteToggleButton
    //

    OexSpriteToggleButton.prototype = Object.create(Sprite_Clickable.prototype);
    OexSpriteToggleButton.prototype.constructor = OexSpriteToggleButton;

    OexSpriteToggleButton.prototype.initialize = function(picture, frames) {
        Sprite_Clickable.prototype.initialize.call(this);
        this._state = 0;
        this._old_state = 0;
        this._frames = frames;
        this.bitmap = ImageManager.loadPicture(picture);
        this.setupFrames();
    };

    OexSpriteToggleButton.prototype.setupFrames = function() {
        const frame = this._frames[this._state];
        if (frame) {
            this.setFrame(frame.x, frame.y, frame.width, frame.height);
        }
    };

    OexSpriteToggleButton.prototype.update = function() {
        Sprite_Clickable.prototype.update.call(this);
        this.updateFrame();
        this.updateOpacity();
        this.processTouch();
    };

    OexSpriteToggleButton.prototype.updateFrame = function() {
        if (this._state != this._old_state) {
            this._old_state = this._state;
            const frame = this._frames[this._state];
            if (frame) {
                this.setFrame(frame.x, frame.y, frame.width, frame.height);
            }
        }
    };

    OexSpriteToggleButton.prototype.updateOpacity = function() {
        //this.opacity = this._pressed ? 255 : 192;
    };

    OexSpriteToggleButton.prototype.setClickHandler = function(method) {
        this._clickHandler = method;
    };

    OexSpriteToggleButton.prototype.onClick = function() {
        this._state += 1;
        this._state = this._state % this._frames.length;
        OexUtils.playSound("__button45");
        if (this._clickHandler) {
            this._clickHandler();
        }
    };


})();
