

(() => {
    const pluginName = "plugin_walk";

Game_Map.prototype.checkPassage = function(x, y, bit) {
    const flags = this.tilesetFlags();
    const tiles = this.allTiles(x, y);
    for (const tile of tiles) {
        const flag = flags[tile];
        if ((flag & 0x10) !== 0) {
            // [*] No effect on passage
            if ((flag & bit) === bit) {
                return false;
            }
            continue;
        }
        if ((flag & bit) === 0) {
            // [o] Passable
            return true;
        }
        if ((flag & bit) === bit) {
            // [x] Impassable
            return false;
        }
    }
    return false;
};

Game_Player.prototype.canPassNotEvent = function(x, y, d) {
    var x2 = $gameMap.roundXWithDirection(x, d);
    var y2 = $gameMap.roundYWithDirection(y, d);
    if (!$gameMap.isValid(x2, y2)) {
        return false;
    }
    if (this.isThrough() || this.isDebugThrough()) {
        return true;
    }
    if (!this.isMapPassable(x, y, d)) {
        return false;
    }
    //if (this.isCollidedWithCharacters(x2, y2)) {
    //    return false;
    //}
    return true;
};


Game_Player.prototype.moveStraightEx = function(direct, goalX, goalY) {
    //console.log("moveStraightEx direct=" + direct);
    if (this.canPassNotEvent(this.x, this.y, direct)) {
        this.moveStraight(direct);
        return;
    }
    if (Math.abs(this.x - goalX) <= 1 && Math.abs(this.y - goalY) <= 1) {
        this.moveStraight(direct);
        return;
    }
    switch (direct) {
    case 2:
        if (this.canPassDiagonally(this.x, this.y, 6, 2)) {
            this.moveDiagonally(6, 2);
        } else if (this.canPassDiagonally(this.x, this.y, 4, 2)) {
            this.moveDiagonally(4, 2);
        } else {
            this.moveStraight(direct);
            return;
        }
        break;
    case 4:
        if (this.canPassDiagonally(this.x, this.y, 4, 2)) {
            this.moveDiagonally(4, 2);
        } else if (this.canPassDiagonally(this.x, this.y, 4, 8)) {
            this.moveDiagonally(4, 8);
        } else {
            this.moveStraight(direct);
            return;
        }
        break;
    case 6:
        if (this.canPassDiagonally(this.x, this.y, 6, 2)) {
            this.moveDiagonally(6, 2);
        } else if (this.canPassDiagonally(this.x, this.y, 6, 8)) {
            this.moveDiagonally(6, 8);
        } else {
            this.moveStraight(direct);
            return;
        }
        break;
    case 8:
        if (this.canPassDiagonally(this.x, this.y, 6, 8)) {
            this.moveDiagonally(6, 8);
        } else if (this.canPassDiagonally(this.x, this.y, 4, 8)) {
            this.moveDiagonally(4, 8);
        } else {
            this.moveStraight(direct);
            return;
        }
        break;
    }
    this.setDirection(direct);
};

Game_Player.prototype.moveDiagonallyEx = function(horz, vert) {
    if (this.canPassDiagonally(this.x, this.y, horz, vert)) {
        this.moveDiagonally(horz, vert);
    } else if (this.canPass(this.x, this.y, horz)) {
        this.moveStraight(horz);
    } else {
        this.moveStraight(vert);
    }
};

Game_Player.prototype.findDirectionTo = function(goalX, goalY) {
    if (this.x < goalX) {
        if (this.y < goalY) {
            // ROUTE_MOVE_LOWER_R:
            this.moveDiagonallyEx(6, 2);
        } else if (this.y > goalY) {
            // ROUTE_MOVE_UPPER_R:
            this.moveDiagonallyEx(6, 8);
        } else {
            // ROUTE_MOVE_RIGHT:
            this.moveStraightEx(6, goalX, goalY);
        }
    } else if (this.x > goalX) {
        if (this.y < goalY) {
            // ROUTE_MOVE_LOWER_L:
            this.moveDiagonallyEx(4, 2);
        } else if (this.y > goalY) {
            // ROUTE_MOVE_UPPER_L:
            this.moveDiagonallyEx(4, 8);
        } else {
            // ROUTE_MOVE_LEFT:
            this.moveStraightEx(4, goalX, goalY);
        }
    } else {
        if (this.y < goalY) {
            // ROUTE_MOVE_DOWN
            this.moveStraightEx(2, goalX, goalY);
        } else if (this.y > goalY) {
            // ROUTE_MOVE_UP
            this.moveStraightEx(8, goalX, goalY);
        } else {
            // do nothing
        }
    }
    return 0;
};

var soundOfStep = function(tag) {
    if ($gamePlayer.isOnLadder()) {
        //OexUtils.playSound("__ladder00");
        return;
    }
    switch (Number(tag)) {
      case 1: // 雲
        OexUtils.playSound("__walk_room00");
        return;
      case 2: // 氷
        OexUtils.playSound("__walk_stone00");
        return;
      case 3: // 木の床
        OexUtils.playSound("__walk_floor00");
        return;
      case 4: // カーペット
        OexUtils.playSound("__walk_room00");
        return;
      case 5: // 雪
        OexUtils.playSound("__walk_snow00");
        return;
      case 6: // 硬い床
        OexUtils.playSound("__walk_stone00");
        return;
    }
    OexUtils.playSound("__walk_land00");
};

var Game_Party_increaseSteps_org = Game_Party.prototype.increaseSteps;
Game_Party.prototype.increaseSteps = function() {
    Game_Party_increaseSteps_org.apply(this, arguments);
    if ($gameSwitches.value(11)) {
        var oldValue = $gameVariables.value(4);
        $gameVariables.setValue(4, oldValue + 2);
    }
};

var Game_Player_increaseSteps_org = Game_Player.prototype.increaseSteps;
Game_Player.prototype.increaseSteps = function() {
  var __steps = $gameParty._steps;
  Game_Player_increaseSteps_org.apply(this, arguments);
  $gameParty._steps = __steps + 1;
  if ($gameParty._steps % 2 == 0) {
    if ($gamePlayer.isInVehicle()) {
    } else {
      var tag = this.terrainTag();
      soundOfStep(tag);
    }
  }
};



})();
